"""Remote SDK objects structure"""
from typing import Literal

from luna3.public.sdk import *  # pylint: disable-msg=W0401,W0614


@dataclass
class ROI(BaseClass):
    """
    Region of interest detection coordinates Model

    Attributes:
        x: Region of interest *x* coordinate
        y: Region of interest *y* coordinate
        height: Region of interest height
        width: Region of interest width
    """

    x: int | float | None  # pylint: disable-msg=C0103
    y: int | float | None  # pylint: disable-msg=C0103
    height: int | float | None
    width: int | float | None


@dataclass
class Estimation(BaseClass):
    """Estimation state base model"""

    estimate: int | None


@dataclass
class DeepfakeEstimation(Estimation):
    """Deepfake estimation state model"""

    scoreThreshold: float | None


@dataclass
class BaseEstimations(BaseClass):
    """Estimations model"""

    warp: Estimation | None  # Warp estimation
    descriptor: Estimation | None  # Descriptor estimation
    detection: Estimation | None  # Detection estimation


@dataclass
class FaceEstimations(BaseEstimations):
    """Face estimations model"""

    basicAttributes: Estimation | None  # Basic Attributes estimation
    deepfake: DeepfakeEstimation | None  # Deepfake estimation
    detectionBackground: Estimation | None  # Detection background estimation
    dynamicRange: Estimation | None  # Dynamic range estimation
    emotions: Estimation | None  # Emotions estimation
    eyebrow: Estimation | None  # Eyebrow estimation
    eyes: Estimation | None  # Eyes estimation
    fisheye: Estimation | None  # Fisheye estimation
    gazeDirection: Estimation | None  # Gaze direction estimation
    glasses: Estimation | None  # Glasses estimation
    headPose: Estimation | None  # Head pose estimation
    headwear: Estimation | None  # Headwear estimation
    landmarks5: Estimation | None  # Basic 5-point facial landmarks estimation
    landmarks68: Estimation | None  # Basic 68-point facial landmarks estimation
    liveness: Estimation | None  # Liveness estimation
    mask: Estimation | None  # Mask estimation
    mouth: Estimation | None  # Mouth estimation
    naturalLight: Estimation | None  # Natural light estimation
    portraitStyle: Estimation | None  # Portrait style estimation
    redEyes: Estimation | None  # Red eyes estimation
    warpQuality: Estimation | None  # Warp quality estimation


@dataclass
class BodyEstimations(BaseEstimations):
    """Body estimations model"""

    attributes: Estimation | None  # Attributes estimation
    accessories: Estimation | None  # Accessories estimation
    landmarks17: Estimation | None  # Landmarks estimation


@dataclass
class TrackProperty(BaseClass):
    """Track property model"""

    minLength: int | None


@dataclass
class BestshotFilters(BaseClass):
    """
    Bestshot model for filtering more acceptable frames
    Attributes:
        ags: ags filter
        maskStates: mask state filter
        pitchThreshold: pitch threshold filter
        rollThreshold: roll threshold filter
        yawThreshold: yaw threshold filter
    """

    ags: float | None
    maskStates: list[Literal[1, 2, 3]] | None
    pitchThreshold: int | None
    rollThreshold: int | None
    yawThreshold: int | None


@dataclass
class FaceAnalyticsParameters(BaseClass):
    """Face analytics parameters"""

    aggregationCount: int | None  # Aggregation count parameter
    estimations: FaceEstimations | None  # Required estimations
    bestshotFilters: BestshotFilters | None  # Bestshot filters


@dataclass
class BodyAnalyticsParameters(BaseClass):
    """Body analytic parameters"""

    aggregationCount: int | None  # Aggregation count parameter
    estimations: BodyEstimations | None  # Required estimations


@dataclass
class AnalyticsParameters(BaseClass):
    """
    Analytics parameters model
    Attributes:
        track: Track parameters such as minimal length
        roi: Region of interest coordinates. Might be or absolute, either in percentage
        droi: Digital region of interest coordinates. Might be or absolute, either in percentage
        face: Face analytics parameters including required estimations, aggregation count and best shot filter
        body: Body analytics parameters including required estimations and aggregation count
    """

    track: TrackProperty | None
    roi: ROI | None
    droi: ROI | None
    face: FaceAnalyticsParameters | None
    body: BodyAnalyticsParameters | None


@dataclass
class VideoSDKAnalytics(BaseClass):
    """SDK Analytics parameters model"""

    name: str | None
    parameters: AnalyticsParameters | None


@dataclass
class VideoSDKProperties(BaseClass):
    """Video url"""

    url: str | None


@dataclass
class VideoSDKInput(BaseClass):
    """Video url"""

    video: VideoSDKProperties | None
    analytics: VideoSDKAnalytics | None
